fl

{
RTC DS3231 driver  
     
 
PropForth 5.5
2013/11/08 7:29:21

   RTC DS1337     Propeller
      scl    ----  P28   
      sda    ----  P29 
   SQW/INT   ----  Pull-up resister ---
}

\ =========================================================================== 
\ Constants 
\ =========================================================================== 
\ Slave addres for DS3231
hD0 wconstant DS3231

\ register name
0 wconstant current
7 wconstant alarm1
9 wconstant alm1_Hour
hA wconstant alm1_DY/DT
hB wconstant alarm2
hC wconstant alm2_Hour
hD wconstant alm2_DY/DT
hE wconstant control
hF wconstant status
h11 wconstant temp

\ register value
h40 wconstant 12Hour
h20 wconstant AM/PM
h40 wconstant DY/DT
h20 wconstant conv

\ SQW_rate
0 wconstant 1Hz     \ 1Hz
1 wconstant 1kHz    \ 1.024kHz
2 wconstant 4kHz    \ 4.096kHz
3 wconstant 8kHz    \ 8.192kHz

\ =========================================================================== 
\ String 
\ =========================================================================== 
\ Convert week-number to string
\ rtc_week ( n1 -- )  n1:1 to 7
: rtc_week
7 and 
c" MONTUEWEDTHUFRISATSUN" 1+ swap 1 max 1- 3 u* +
3 0 do dup C@ emit 1+ loop drop 
;

\ Convert month-number to string
\ rtc_mon ( n1 -- )  n1:1 to 12
: rtc_mon 
c" JANFEBMARAPRMAYJUNJULAUGSEPOCTNOVDEC" 1+ swap hC min 1 max 1- 3 u* +
3 0 do dup C@ emit 1+ loop drop 
;

\ =========================================================================== 
\ BCD Conversion
\ =========================================================================== 
\ bcd> ( n1 -- n2 ) convert bcd byte n1 to hex byte n2
[ifndef bcd>
: bcd>
	dup hF and
	swap hF0 and
	1 rshift dup
	2 rshift + +
;
]

\
\ >bcd ( n1 -- n2 ) convert hex byte n1 to bcd byte n2
[ifndef >bcd
: >bcd
	d10 u/mod 4 lshift +
;
]

\ _rtcflip7 ( n7 n6 n5 n4 n3 n2 n1 -- n1 n2 n3 n4 n5 n6 n7 ) flip the top 7 items on the stack
: _rtcflip7 
	rot >r rot >r rot >r rot >r rot >r
	swap
	r> r> r> r> r>
	rot >r rot >r rot >r
	swap
	r> r> r>
	swap rot
;

\ =========================================================================== 
\  Word for I2C_device
\ =========================================================================== 
: err_msg ." I2C error" ;
\ If error, print message
\ ( n1 -- )   n1:t/f
: err? if err_msg cr then ;

\ Read data from register in i2c_device
\ ( n1 n2 -- n3 t/f )  n1:register  n2:slave_address  n3:data  t/f:true if there was an error
: i2c_rd
\ Start I2C 
_eestart
\ Write slave address[wr], then receive Acknowledge-bit(ACK:Lo  NACK:Hi)   
tuck _eewrite                 \ ( n2 n1 t/f )
\ Write register
swap _eewrite or              \ ( n2 t/f )
swap                          \ ( t/f n2 )
\ Start read_process
_eestart
\ Write slave address[rd], then receive Acknowledge-bit(ACK:Lo  NACK:Hi)
1 or _eewrite or              \ ( t/f )
\ Read 1byte ,then set sda to Hi(NACK:master->slave)
-1 _eeread             
\ Stop I2C
 _eestop                                         
 swap                         \ (n3 t/f )
;

\ Write data to register in i2c_device
\ ( n1 n2 n3 -- t/f )  n1:data  n2:register  n3:slave_address  t/f:true if there was an error
: i2c_wr
\ Start I2C 
_eestart
\ Write slave address[wr], then receive Acknowledge-bit(ACK:Lo  NACK:Hi) 
_eewrite                      \ ( n1 n2 t/f )
\ Write register
swap _eewrite or              \ ( n1 t/f )
\ Write data   
swap _eewrite or              \ ( t/f )   
\ Stop I2C
 _eestop 
;

\ =========================================================================== 
\ Reading register value
\ =========================================================================== 
\ Display all register's value 
: disp_reg
." Address(hex)  " ." value(hex)" cr
hex
h13 0 do
     9 emit
     i .byte                  \ Display register address
     9 emit
     i DS3231 i2c_rd err?     \ Read value
     .byte                    \ Display register value
     cr
loop
decimal
;

\ =========================================================================== 
\ Interrupt control for alarm1/alarm2
\ =========================================================================== 

\ Set/Clear INTCN bit
\ ( n1 -- )  n1:0/1  0=clear 1=set    
: set_INTCN 
control        \ register 
DS3231         \ slave address
i2c_rd         \ ( n1 data t/f )
rot2           \ ( t/f n1 data )
hFB and swap 
if 4 or then                  \ ( t/f data )
control                       \ register
DS3231                        \ slave address
i2c_wr                        \ ( t/f t/f )
or err?
;

: INT_msg and if ." Enabled" else ." Disabled" then cr ;

\ Display alarm1/alarm2 Interupt status
\ ( -- )
: chk_IE 
control        \ register
DS3231         \ slave address
i2c_rd         \ ( data t/f )
if
     err_msg drop cr
else
     dup 
     ." Alarm1 Interupt:" 1 INT_msg 
     ." Alarm2 Interupt:" 2 INT_msg 
then
;

\ Enable/Disable interupt for alarm1/alarm2
\ ( n1 n2 -- t/f )  n1:enable[1] disable[0]  n2:alarm1 or alarm2  t/f:true if there was an error
: set_IE
control        \ register
DS3231         \ slave address
i2c_rd         \ ( n1 n2 data t/f )
if
     3drop 1
else
     rot2      \ ( data n1 n2 )
     alarm1 =
     if 
          \ alarm1
          if 1 or else hFE and then     \ Set or Clear bit0[A1IE]  
     else
          \ alarm2
          if 2 or else hFD and then     \ Set or Clear bit1[A2IE]
     then           \ data
     control        \ register
     DS3231         \ slave address
     i2c_wr
     chk_IE
then
err?
;

: status_msg and if ." 1" else ." 0" then cr ;

\ Display alarm flag
\ ( -- )
: chk_INT
status         \ register
DS3231         \ slave address
i2c_rd         \ ( data t/f )
if
     err_msg drop cr
else
     dup
     ." Alarm1 Flag:" 1 status_msg
     ." Alarm2 Flag:" 2 status_msg cr
then
;

\ Clear INT-flag for alarm1/alarm2, then clear A1M4 and A2M3 (Alarm when hour.minutes match) 
\ ( n1 -- )  n1:alarm1 or alarm2   
: clr_INT
status         \ register
DS3231         \ slave address
i2c_rd         \ ( n1 data t/f )
if
     err_msg 2drop
else
     swap                     \ ( data n1 )
     alarm1 = 
     if 
          \ Clear bit0
          hFE and             \ ( data*hFE )       
          \ Clear A1M4
          alm1_DY/DT dup DS3231 i2c_rd      \ ( n1*hFE register data t/f )
          if
               err_msg 2drop 1
          else
               h7F and swap DS3231 i2c_wr   \ ( n1*hFE t/f )   
          then
     else 
          \ clear bit1
          hFD and             \ ( data*hFD )                
          \ Clear A2M4
          alm2_DY/DT dup DS3231 i2c_rd      \ ( n1*hFDE register data t/f )
          if
               err_msg 2drop 1
          else     
               h7F and swap DS3231 i2c_wr   \ ( n1*hFE t/f ) 
          then
     then
     if
          err_msg drop
     else
          status DS3231 i2c_wr err?
     then
then
chk_INT
;
                
\ =========================================================================== 
\ Manipulate digits
\ =========================================================================== 
\ Print 1digit
\ ( n1 -- ) n1:digit (0-9)
: 1digit tochar emit ;

\ Alignment/Print time-digit to 2digits
\ ( n1 -- ) n1:time-digit (0-24,0-59)
: 2digit
dup d10 <
if
     \ If n1 < d10
     0 tochar emit tochar emit
else
     \ If n1 > 9
     d10 u/mod tochar emit tochar emit
then
;

\ =========================================================================== 
\ Manipulate 12/24-mode
\ =========================================================================== 
\ Set 12/24-mode to current/alarm1/alarm2
\ ( n1 n2 -- )  n1:1=12Hour or 0=24Hour  n2:register[current or alarm1 or alarm2]
: set_12/24
dup alarm2 = if 1+ else 2+ then              \ Get Hour regiter address
                                             \ ( n1 n2 )
2dup                                         \ ( n1 n2 n1 n2 )
DS3231 i2c_rd                                \ Read Hour register  ( n1 n2 n1 data t/f )
err?                                         \ ( n1 n2 n1 data )
12Hour and 6 rshift <>                       \ ( n1 n2 t/f )
if
                                             \ ( n1 n2 )
     dup DS3231 i2c_rd err?                  \ ( n1 n2 data )
     dup 12Hour and 0 =                                             
     if
          \ Change 24Hour to 12Hour
          bcd> dup d12 > 
          if d12 - 1 else 0 then             \ If hour > 12, subtract d12, then set [AM/PM]
          5 lshift swap >bcd or 12Hour or    \ Add bit6(12Hour) +bit5(PM/AM) + Hour(bit4-bit0)
     else
          \ Change 12Hour to 24Hour 
          dup AM/PM and 0= if 0 else 1 then  \ Check if AM/PM
          swap h1F and bcd> swap             
          if d12 + then                      \ Add d12 if PM
          >bcd
     then                                    \ ( n1 n2 data )
     over DS3231 i2c_wr
     err?                                    \ ( n1 n2 )
then
2drop
;

\ =========================================================================== 
\ Common word on current, alarm1, alarm2
\ =========================================================================== 
\ Display AM or PM checking AM/PM on 12Hour mode
\ ( n1 -- n2 )  n1:Hour register(bcd)  n2:Hore(binary)
: disp_AM/PM
dup AM/PM and swap            \ Check AM/PM 
h1F and bcd> swap             \ Convert Hour to hex byte
if ." PM" else ." AM" then    \ Print AM or PM
space
;

\ Display AM/PM and calculate Hour-time if 12Hour mode
\ ( n1 -- n2 )  n1:Hour register(binary)  n2:Hore(binary)
: chk_12H_mode
." Time: "
>bcd dup 12Hour and 0<>
if 
     disp_AM/PM          \ Display AM/PM on 12Hour mode
else 
     bcd>                \ Convert data to binary on 24Hour mode 
then
;

\ Display or Not display Day/date 
\ ( n1 n2 -- )   n1:Hour  n2:DAY/DT register  
: disp_DY/DT
DS3231 i2c_rd                                          \ Get Day/Date for alarm1 or alarm2
err?                                                   \ ( n1 data )
dup h80 and 0=                                         \ Check A1M4/A2M4 whether display  DY/DT or not
if
     DY/DT and                                         \ Day or Date?
     if ." Week: " rtc_week else ." Date: " . then cr  \ Display Week or Date   
else
     2drop     
then                                                   \ ( n1 )
chk_12H_mode                                           \ Display AM/PM if 12Hour mode   
;

\ Convert input-value to bcd
\ ( n1 n2 -- n3 )  n1:data n2:Hour register    n3:Hour 
: conv_hour
DS3231 i2c_rd                           \ Get Hour
err?                                    \ ( data )
dup 12Hour and                          
\ Check if 12Hour-mode 
if 
                                        \ ( data )
     \ 12Hour mode
     swap dup d12 > 
     if 
          \ > d12
          d12 - >bcd AM/PM or           \ Add bit5[AM/PM]
     else
          \ d13 >
          >bcd                         
     then
     12Hour or swap                     \ Add bit6[12/24]
else
     \ 24Hour mode
     swap >bcd swap
then
h80 and                                 \ mask bit7[A1m4 or A2M4]
or                                      \ bit7[1 or 0] + Hour  
;

\ =========================================================================== 
\ Current time
\ =========================================================================== 
\ Get current time 
\ Read/Convert current time from DS3231
\ ( -- n1 n2 n3 n4 n5 n6 n7 )
\ n1 - second		(00 - 59)
\ n2 - minute		(00 - 59)
\ n3 - hour		(00 - 23)
\ n4 - day of week 	(Mon:1 Tue:2 Wed:3 Thur:4 Fri:5 Sat:6 San:7)
\ n5 - date		(01 - 31)
\ n6 - month		(01 - 12)
\ n7 - yesr		(2000 - 2099)   
: rd_current
7 0 do current i + DS3231 i2c_rd err? bcd> loop                 
cr   ." CurrentTime" cr
." Year: " d2000 + . cr
." Date: " rtc_mon space . rtc_week cr
chk_12H_mode
2digit h3A emit 2digit h3A emit 2digit  
cr cr    
;

\ Set current-time to DS3231   (24Hour mode)
\ Set second to 0
\ ( n1 n2 n3 n4 n5 n6 -- ) 
\ n1 - yesr		(00 - 99)   
\ n2 - month		(01 - 12)
\ n3 - date		(01 - 31)
\ n4 - day-of-week  (Mon:1 Tue:2 Wed:3 Thur:4 Fri:5 Sat:6 San:7)
\ n4 - hour		(00 - 23)
\ n5 - minute		(00 - 59)
: set_current 
0 status DS3231 i2c_wr        \ Clear OSF
err?
>bcd >r                       \ minute
>bcd >r                       \ hour
>r                            \ day-of-week
>bcd >r                       \ day
>bcd >r                       \ month
d2000 - >bcd                  \ year
r> r> r> r> r> 0     
7 0 do current i + DS3231 i2c_wr err? loop
rd_current 
0 alarm1 set_12/24 0 alarm2 set_12/24        \ Set alarm1/alarm2 to 24Hour mode

\ Clear A1M1 
alarm1 DS3231 i2c_rd err? h7F and alarm1 DS3231 i2c_wr err?
\ Clear A1M3 
alm1_Hour DS3231 i2c_rd err? h7F and alm1_Hour DS3231 i2c_wr err?
\ Clear A2M3
alm2_Hour DS3231 i2c_rd err? h7F and alm2_Hour DS3231 i2c_wr err?
\ Clear A2M4 
alm2_DY/DT DS3231 i2c_rd err? h7F and alm2_DY/DT DS3231 i2c_wr err?
;

\ =========================================================================== 
\ Alarm1 time
\ =========================================================================== 
\ Get alarm1 time from DS3231
\ ( -- n1 n2 n3 n4 )
\ n1 - second		(00 - 59)
\ n2 - minute		(00 - 59)
\ n3 - hour		(00 - 23)
\ n4 - day of week 	(Mon:1 Tue:2 Wed:3 Thur:4 Fri:5 Sat:6 San:7)
\      date  (1-31)
: rd_alm1
4 0 do alarm1 i + DS3231 i2c_rd err? bcd> loop            
cr  ." Alarm1 setting" cr                                     
\ Check if Day or Date or Ignore, then 12Hour-mode                     
alm1_DY/DT disp_DY/DT
2digit h3A emit 2digit h3A emit 2digit  cr cr     
;

\ Set alarm1-time to DS3231   (24Hour mode)
\ Set second to 0
\ ( n1 n2 n3 -- ) 
\ n1 - date		(01 - 31)       bit[DY/DT]=0
\ n1 - day-of-week  (Mon:1 Tue:2 Wed:3 Thur:4 Fri:5 Sat:6 San:7)    bit[DY/DT]=1
\ n2 - hour		(00 - 23)
\ n3 - minute		(00 - 59)
: set_alm1 
>bcd >r                       \ minute
>r                            \ hour
>bcd >r                       \ day / day-of-week
r> r> r> 0                                                              
2 0 do alarm1 i + DS3231 i2c_wr err? loop    \ Save second and minute to register
\ Hour   
alm1_Hour conv_hour                                                     
alm1_Hour DS3231 i2c_wr                      \ Save Hour to register          
err? 
alm1_DY/DT DS3231 i2c_rd err? 
DY/DT h80 or and or 
alm1_DY/DT DS3231 i2c_wr err?                \ Save bit[A1M4] + bit[Day or Date] + InputValue to register
rd_alm1                                      \ Display alarm1 setting
\ Interrupt Enable
1 alarm1 set_IE
;

\ =========================================================================== 
\ Alarm2 time
\ =========================================================================== 
\ Get alarm2 time from DS3231
\ ( -- n1 n2 n3 )
\ n1 - minute		(00 - 59)
\ n2 - hour		(00 - 23)
\ n3 - day of week 	(Mon:1 Tue:2 Wed:3 Thur:4 Fri:5 Sat:6 San:7)
\      date
: rd_alm2
3 0 do alarm2 i + DS3231 i2c_rd err? bcd> loop
cr  ." Alarm2 setting" cr
\ Check if Day or Date or Ignore, then 12Hour-mode                     
alm2_DY/DT disp_DY/DT
2digit h3A emit 2digit cr cr     
;

\ Set alarm2-time to DS3231   (24Hour mode)
\ ( n1 n2 n3 -- ) 
\ n1 - date		(01 - 31)         bit[DY/DT]=0
\ n1 - day-of-week  (Mon:1 Tue:2 Wed:3 Thur:4 Fri:5 Sat:6 San:7)   bit[DY/DT]=1
\ n2 - hour		(00 - 23)
\ n3 - minute		(00 - 59)
: set_alm2 
>bcd >r                       \ minute
>r                            \ hour
>bcd >r                       \ day / day-of-week
r> r> r>      
alarm2 DS3231 i2c_wr err?     \ Save minute to register
\ Hour
alm2_Hour conv_hour
alm2_Hour DS3231 i2c_wr err?  \ Save Hour to register          
 
alm2_DY/DT DS3231 i2c_rd err? 
DY/DT h80 or and or 
alm2_DY/DT DS3231 i2c_wr err?                \ Save bit[A2M4] + bit[Day or Date] + InputValue to register
rd_alm2                                      \ Display alarm1 setting

\ Interrupt Enable
1 alarm2 set_IE
;
                                                                       
\ =========================================================================== 
\ Manipulating special-purpose registers for osc
\ =========================================================================== 
\ Enable/Disable 32.768kHz from 32kHz-pin(1)
\ ( n1 -- )  n1:0(disable)/1(enable)
: 32kHz_out
hF DS3231 i2c_rd err?
2dup                     \ ( n1 data n1 data )
8 and 3 rshift <>        \ ( n1 data t/f )
if
     swap                \ ( data n1 )
     if
          \ Output 32.768kHz
          8 or hF DS3231 i2c_wr err?
     else
          \ Stop 32.768kHz
          hF7 and hF DS3231 i2c_wr err?
     then
else
     swap                \ ( data n1 )
     if
          ." 32.768kHz already is enabled."     
     else
          ." 32.768kHz is disabled."
     then
     cr
then
;

\ Set SQW
\ ( n1 --)  n1:1Hz or 1kHz(1.042kHz) or 4kHz(4.096kHz) or 8kHz(8.192kHz) (default:working as INT)
: set_SQW 
control DS3231 i2c_rd err? 
hE7 and swap 3 lshift or 
control DS3231 i2c_wr err? 
;

\ Enable\Disable osc when battery backup
\ When DS3231 is powered by Vcc, osc is always on regardless of the status of the EOSC bit
\ ( n1 -- ) n1:0(disable)/1(enable)
: EOSC
control DS3231 i2c_rd err? swap         \ ( data n1 )
if
     \ enable osc when battery backup    ( data )
     h7F and 
else
     \ disable osc when battery backup   ( data )
     h80 or 
then
control DS3231 i2c_wr err?
;

\ Read OSC-status
\ ( -- )  n1:0(stop)/1(start)
: osc_state
status DS3231 i2c_rd err?
h80 and
if
     ." Oscillator is stopped or was stopped for some period." cr
     status DS3231 i2c_rd err? h7F and status DS3231 i2c_wr err? 
     ." Oscillator start"
else
     ." Oscillator operate"
then
cr
;

\ =========================================================================== 
\ Convert temperature
\ =========================================================================== 
\ Get Temperature
\ ( -- )
: get_Temp
control DS3231 i2c_rd err?    \ Read control register 
h20 or 
control DS3231 i2c_wr err?    \ Write setting CONV-bit to control register 
h11 DS3231 i2c_rd err?             
. h2E emit
h12 DS3231 i2c_rd err?             
h40 over =
if h32 emit h35 emit
else h80 over =
if h35 emit
else hC0 over =
if h37 emit h35 emit
else h30 emit
thens
drop 
." degree" cr
;
  
: disp_Temp
begin
     \ Check if conversion is ready
     begin status DS3231 i2c_rd err? 4 and 0= until
     \ Get temperature  
     get_Temp cr
     fkey? swap drop
until
;

\ =========================================================================== 
\ Utility
\ =========================================================================== 
\ Display current, alarm1, alarm2
\ ( -- )
: all rd_current rd_alm1 rd_alm2 ;

\ Set Time(current,alarm1,alarm2) to 12Hour or 24Hour
\ ( n1 -- )  n1:1[12Hour] or 0[24Hour]
: set_12H
dup dup current set_12/24 alarm1 set_12/24 alarm2 set_12/24
rd_current rd_alm1 rd_alm2
;

\ Disable/Enable No Day/Date on alarm1(hour,minute,0second)
\                                       on alarm2(hour,minute)
\ ( n1 n2 -- )  n1:0(disable)/1(enable)  n2:alarm1 or alarm2
: no_DY/DT
alarm1 =
if
     \ alarm1
     if
          \ ignore DY/DT (Set bit[A1M4])
          alm1_DY/DT DS3231 i2c_rd           \ ( data t/f )
          swap h80 or                        \ ( t/f data )
     else
          \ valid DY/DT (Clear bit[A1M4])    
          alm1_DY/DT DS3231 i2c_rd           \ ( data t/f )
          swap h7F and                       \ ( t/f data )
     then
     alm1_DY/DT                              \ ( t/f data register )
else
     \ alarm2
     if
          \ ignore DY/DT (Set bit[A2M4]) 
          alm2_DY/DT DS3231 i2c_rd           \ ( data t/f )
          swap h80 or                        \ ( t/f data )
     else
          \ valid DY/DT (Clear bit[A2M4]) 
          alm2_DY/DT DS3231 i2c_rd           \ ( data t/f )
          swap h7F and                       \ ( t/f data )
     then
     alm2_DY/DT                              \ ( t/f data register )
then
DS3231 i2c_wr                                \ ( t/f t/f )
or err?
;

\ Set DY/DT-mode on alarm1 and alarm2
\ If changed DY/DT-mode, value for Day/Date-mode must be reinitialized
\ ( n1 n2 -- ) n1:1=Day  0=Date   n1:alarm1 or alarm2
: alm_Day
alarm1 = if alm1_DY/DT else alm2_DY/DT then       \ ( n1 register ) 
tuck                                              \ ( register n1 register )
DS3231 i2c_rd                                     \ ( register n1 data t/f )
swap rot                                          \ ( register t/f data n1 )
if 
     DY/DT or            \ Set bit[DY/DT] 
else 
     DY/DT invert and    \ Clear bit[DY/DT] 
then                                              \ ( register t/f data )
rot                                               \ ( t/f data register )
DS3231 i2c_wr                                     \ ( t/f t/f )
or err?
;

